# Stubs for pysolar.elevation (Python 3.6)

def get_pressure_with_elevation(h:float, Ps:float = ..., Ts:float = ..., Tl:float = ..., Hb:float = ..., R:float = ..., g:float = ..., M:float = ...) -> float: ...
def get_temperature_with_elevation(h:float, Ts:float = ..., Tl:float = ...) -> float: ...
def elevation_test() -> None: ...
