# Stubs for pysolar.radiation (Python 3.6)

import datetime

def get_air_mass_ratio(altitude_deg:float) -> float: ...
def get_apparent_extraterrestrial_flux(day:float) -> float: ...
def get_optical_depth(day:float) -> float: ...
def get_radiation_direct(when:datetime.datetime, altitude_deg:float) -> float: ...
